@@ -44,6 +44,7 @@
 
 #define NETWORK_CONN_TIMEOUT 10
 #define NETWORK_IO_TIMEOUT   30
+#define DEFAULT_TRYMETHOD    "first"
 
 #define SA_NO_SERVER_LEFT 127
 #define SA_SERVER_CONNECT_FAILURE 64
@@ -99,6 +100,7 @@ pam_sm_authenticate( pam_handle_t *pamh, int flags, int argc, const char **argv)
     int conn_timeout;
     char *debuglevel_buf;
     char *blockedfile;
+    char *trymethod;
 
 
     password = NULL;
@@ -137,6 +139,14 @@ pam_sm_authenticate( pam_handle_t *pamh, int flags, int argc, const char **argv)
     log_debug(DEBUG_1, "debuglevel=%d", params.debuglevel);
 #endif
 
+    trymethod = get_config(configfile, "TryMethod");
+    if(trymethod == NULL) {
+        trymethod = DEFAULT_TRYMETHOD;
+    }
+#ifdef DEBUG
+    log_debug(DEBUG_1, "trymethod=%s", trymethod);
+#endif
+
     /********
       Get username
      */
@@ -252,7 +262,7 @@ pam_sm_authenticate( pam_handle_t *pamh, int flags, int argc, const char **argv)
 #endif
         if(result == SA_NO_SERVER_LEFT) {
 #ifdef DEBUG
-            log_debug(DEBUG_5, "auth service cannot retrieve information.");
+            log_debug(DEBUG_5, "No more authentication server cannot be found.");
 #endif
             password = NULL;
             global.password = NULL;
@@ -262,9 +272,18 @@ pam_sm_authenticate( pam_handle_t *pamh, int flags, int argc, const char **argv)
             continue;
         }
         else if(result == SA_SERVER_LOGIN_FAILURE) {
-            password = NULL;
-            global.password = NULL;
-            return PAM_AUTH_ERR;
+#ifdef DEBUG
+            log_debug(DEBUG_5, "authentication error cnt=%d.", cnt);
+#endif
+            if(strncmp(trymethod,"first",5) == 0) {
+                password = NULL;
+                global.password = NULL;
+                return PAM_AUTH_ERR;
+            } else {
+#ifdef DEBUG
+            log_debug(DEBUG_5, "retry next server...");
+#endif
+            }
         } else if(result == PAM_AUTH_ERR) {
             syslog(LOG_ERR, "[pam_smtpauth] authentication error cnt=%d.", cnt);
 #ifdef DEBUG
@@ -352,6 +371,8 @@ smtp_connect(int num) {
 #endif
 
     //sleep(3);
+
+    // Program error
     if(smtp == 0) {
         syslog(LOG_ERR, "[pam_smtpauth] mail_status -> FAIL");
 #ifdef DEBUG
@@ -359,6 +380,8 @@ smtp_connect(int num) {
 #endif
         return PAM_AUTH_ERR;
     }
+
+    // remote connection error or the error before AUTH command(EHLO etc)
     else if(smtp->error == 1) {
         syslog(LOG_WARNING, "[pam_smtpauth] SERVER connection failure: %s:%d => %s", global.host, global.port, smtp->error_message);
 #ifdef DEBUG
@@ -366,6 +389,8 @@ smtp_connect(int num) {
 #endif
         return SA_SERVER_CONNECT_FAILURE;
     }
+
+    // the error after AUTH command
     else if(smtp->error == 2) {
         syslog(LOG_ERR, "[pam_smtpauth] LOGIN FAILURE user %s on %s:%d => %s", global.username, global.host, global.port, smtp->error_message);
 #ifdef DEBUG
@@ -373,6 +398,8 @@ smtp_connect(int num) {
 #endif
         return SA_SERVER_LOGIN_FAILURE;
     }
+
+    // Authentication is complete and OK
     else if(smtp->error == 0) {
         syslog(LOG_INFO, "[pam_smtpauth] mail_status -> OK for %s", global.username);
 #ifdef DEBUG
@@ -380,6 +407,8 @@ smtp_connect(int num) {
 #endif
         return PAM_SUCCESS;
     }
+
+    // Other errors
     else {
         return PAM_CRED_INSUFFICIENT;
     }
